/**
 * Copyright 2003 Sun Microsystems, Inc.
 *
 * See the file "license.terms" for information on usage and
 * redistribution of this file, and for a DISCLAIMER OF ALL
 * WARRANTIES.
 */
import java.io.File;
import java.util.Locale;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;

import javax.speech.Central;
import javax.speech.Engine;
import javax.speech.EngineException;
import javax.speech.synthesis.Synthesizer;
import javax.speech.synthesis.SynthesizerModeDesc;
import javax.speech.synthesis.SynthesizerProperties;
import javax.speech.synthesis.Voice;
import javax.speech.synthesis.Speakable;
import javax.speech.synthesis.SpeakableAdapter;
import javax.speech.synthesis.SpeakableEvent;

/**
 * Sample program that demonstrates how multiple voices
 * and synthesizers can be used.
 */
public class PaymentMachine {

    private final static void usage() {
	System.out.println("PaymentMachine [-showEvents] [-showPropertyChanges]");
    }

    /**
     * Returns a "no synthesizer" message, and asks
     * the user to check if the "speech.properties" file is
     * at <code>user.home</code> or <code>java.home/lib</code>.
     *
     * @return a no synthesizer message
     */
    static private String noSynthesizerMessage(String synthesizer) {
        String message =
            "Cannot find " + synthesizer + ".\n" +
            "This may be the result of any number of problems.  It's\n" +
            "typically due to a missing \"speech.properties\" file that\n" +
            "should be at either of these locations: \n\n";
        message += "user.home    : " + System.getProperty("user.home") + "\n";
        message += "java.home/lib: " + System.getProperty("java.home") +
	    File.separator + "lib\n\n" +
            "Another cause of this problem might be corrupt or missing\n" +
            "voice jar files in the freetts lib directory.  This problem\n" +
            "also sometimes arises when the freetts.jar file is corrupt\n" +
            "or missing.  Sorry about that.  Please check for these\n" +
            "various conditions and then try again.\n";
        return message;
    }

    public static void main(String[] argv) {
	boolean showEvents = false;
	boolean showPropertyChanges = false;

	for (int i = 0; i < argv.length; i++) {
	    if (argv[i].equals("-showEvents")) {
		showEvents = true;
	    } else if (argv[i].equals("-showPropertyChanges")) {
		showPropertyChanges = true;
	    } else {
		usage();
		System.exit(0);
	    }
	}

	System.out.println(" ** Payment Machine - JSAPI Demonstration program **");

        /* kevin16 in a 16khz general domain diphone voice
         */
	Voice kevinHQ = new Voice("kevin16",
		Voice.GENDER_DONT_CARE, Voice.AGE_DONT_CARE, null);

	try {
	    /* Find a synthesizer that has the general domain voice
             * we are looking for.  NOTE:  this uses the Central class
             * of JSAPI to find a Synthesizer.  The Central class
             * expects to find a speech.properties file in user.home
             * or java.home/lib.
             *
             * If your situation doesn't allow you to set up a
             * speech.properties file, you can circumvent the Central
             * class and do a very non-JSAPI thing by talking to
             * FreeTTSEngineCentral directly.  See the WebStartClock
             * demo for an example of how to do this.
             */
	    SynthesizerModeDesc generalDesc = new SynthesizerModeDesc(
		null,          // engine name
                "general",     // mode name
                Locale.US,     // locale
                null,          // running
                null);         // voice

	    final Synthesizer synthesizer1 =
                Central.createSynthesizer(generalDesc);

	    if (synthesizer1 == null) {
		System.err.println(
                    noSynthesizerMessage("general domain synthesizer"));
		System.exit(1);
	    }

	    /* get general domain synthesizer ready to speak
             */
	    System.out.print("Loading voices...");
	    synthesizer1.getSynthesizerProperties().setVoice(kevinHQ);

	    if (showPropertyChanges) {
		synthesizer1.getSynthesizerProperties().addPropertyChangeListener(
                    new PropertyChangeListener() {
                        public void propertyChange(
                            PropertyChangeEvent pce) {
                            if (pce.getNewValue() instanceof Voice) {
				String newVoice =
				    ((Voice) pce.getNewValue()).getName();
				System.out.println(
                                    "  PCE Voice changed to " + newVoice);
			    } else {
				System.out.println(
                                    "  PCE " + pce.getPropertyName()
                                    + " changed from "
				    + pce.getOldValue() + " to " +
				    pce.getNewValue() + ".");
			    }
			}
		    });
	    }

	    if (showEvents) {
		synthesizer1.addSpeakableListener(
		    new SpeakableAdapter() {
			public void markerReached(SpeakableEvent e) {
			    dumpEvent(e);
			}
			public void speakableCancelled(SpeakableEvent e) {
			    dumpEvent(e);
			}
			public void speakableEnded(SpeakableEvent e) {
			    dumpEvent(e);
			}
			public void speakablePaused(SpeakableEvent e) {
			    dumpEvent(e);
			}
			public void speakableResumed(SpeakableEvent e) {
			    dumpEvent(e);
			}
			public void speakableStarted(SpeakableEvent e) {
			    dumpEvent(e);
			}
			public void topOfQueue(SpeakableEvent e) {
			    dumpEvent(e);
			}
			public void wordStarted(SpeakableEvent e) {
			    dumpEvent(e);
			}
			private void dumpEvent(SpeakableEvent e) {
			    System.out.println(" EVT: " + e.paramString()
                                               + " source: " + e.getSource());
			}
		    });
	    }

	    System.out.println("And here we go!");
	    synthesizer1.resume();

	    // Start the payment machine dialog
	    synthesizer1.speakPlainText("Hello! Welcome to the speech-enabled payment machine.", null);
	    System.out.println("Initial Greeting - requesting ticket");
	    synthesizer1.speakPlainText("Please insert your ticket", null);
	    synthesizer1.speakPlainText("Thank you.", null);

	    // Need a delay here for processing
	    synthesizer1.speakPlainText("I'm checking your ticket now. Please wait.", null);
	    System.out.println("Validating ticket");

	    synthesizer1.speakPlainText("Your vehicle was parked for 4 hours.", null);
	    synthesizer1.speakPlainText("Please pay 4 dollars.", null);
	    synthesizer1.speakPlainText("Do you need a receipt?", null);
	    System.out.println("Handle receipt request");
	    // User would press a button here

	    // Need a delay here for processing
	    synthesizer1.speakPlainText("Thank you. Please take your ticket.", null);
	    synthesizer1.speakPlainText("To avoid further charges, you now have 15 minutes to leave the car park.", null);
	    synthesizer1.speakPlainText("Have a nice day.", null);
	    synthesizer1.waitEngineState(Synthesizer.QUEUE_EMPTY);

	    // clean up
	    synthesizer1.deallocate();
	}
	catch (Exception e) {
	    e.printStackTrace();
	}

	System.exit(0);
    }
}

